==> tin-1.02.tar.gz: tin-1.02/signal.c <==
/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : signal.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 10-09-91
 *  Release   : 1.0
 *  Notes     : signal handlers for different modes and window resizing
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

extern char *glob_art_group;
extern char *glob_group;
extern int glob_respnum;
extern char *glob_page_group;

#ifdef SIGTSTP
int do_sigtstp = 0;
#endif

#ifdef POSIX_JOB_CONTROL
struct sigaction art_act;
struct sigaction group_act;
struct sigaction main_act;
struct sigaction page_act;
struct sigaction rcfile_act;
struct sigaction select_act;
struct sigaction old_act;
#endif


void set_signal_handlers ()
{
	signal (SIGINT, signal_handler);		/* ctrl-C */
	signal (SIGQUIT, signal_handler);		/* ctrl-\ */
	signal (SIGILL, signal_handler);
	signal (SIGBUS, signal_handler);
	signal (SIGSEGV, signal_handler);
 	
	signal (SIGPIPE, SIG_IGN);

#ifdef SIGTSTP
	{
		void (*ptr)();
		ptr = signal (SIGTSTP, SIG_DFL);
		signal (SIGTSTP, ptr);
		if (ptr != SIG_IGN) {
			/*
			 *  SIGTSTP is ignored when starting from shells
			 *  without job-control
			 */
			do_sigtstp = 1; 
			signal (SIGTSTP, main_suspend);
		}
	}
#endif

#ifdef SIGWINCH
	signal (SIGWINCH, main_resize);
#endif
}


void signal_handler (sig)
	int sig;
{
	switch (sig) {
		case SIGINT:
			signal (SIGINT, signal_handler);
			break;
		default:
			Raw (FALSE);
			printf ("\n%s: signal handler caught signal %d\n", progname,sig);
			exit (1);
	}
}


void set_win_size (num_lines, num_cols)
	int *num_lines;
	int *num_cols;
{
#ifdef TIOCGWINSZ
	char buf[64];
	int old_lines, old_cols;
	struct winsize win;

	if (debug) {
		old_lines = *num_lines;
		old_cols = *num_cols;
	}
#endif

	init_screen_array (FALSE);		/* deallocate screen array */

#ifdef TIOCGWINSZ
	if (ioctl (0, TIOCGWINSZ, &win) == 0) {
		if (win.ws_row != 0) {
			*num_lines = win.ws_row - 1;
		}
		if (win.ws_col != 0) {
			*num_cols = win.ws_col;
		}
	}

	if (debug) {
		sprintf (buf, "RESIZED lines %d to %d  cols %d to %d",
			old_lines, *num_lines, old_cols, *num_cols);
		info_message (buf);
	}
#endif

	init_screen_array (TRUE);		/* allocate screen array for resize */

	max_subj = (*num_cols / 2) - 2;
	max_from = (*num_cols - max_subj) - 17;
	RIGHT_POS = *num_cols - 18;
	MORE_POS  = *num_cols - 15;
	NOTESLINES = *num_lines - INDEX_TOP - 1;
	if (NOTESLINES <= 0) {
		NOTESLINES = 1;
	}
}


#ifdef SIGTSTP

/* ARGSUSED0 */
void art_suspend (sig)
	int sig;
{
	char buf[LEN];
	
	Raw (FALSE);
	putchar ('\n');

#ifdef POSIX_JOB_CONTROL
	sigsetmask(0);
#else
	signal(SIGTSTP, SIG_DFL);
#ifdef BSD
	sigsetmask (sigblock(0) & ~(1 << (SIGTSTP -1)));
#endif
#endif

	kill (0, SIGTSTP);

#ifdef POSIX_JOB_CONTROL
	sigemptyset (&art_act.sa_mask);
	art_act.sa_flags = SA_RESTART | SA_RESETHAND;
	art_act.sa_handler = art_suspend;
	sigaction (SIGTSTP, &art_act, 0L);
#else
	signal (SIGTSTP, art_suspend);
#endif

	if (! update) {
		mail_setup ();
	
		Raw (TRUE);

		ClearScreen ();
		sprintf (buf, txt_group, glob_art_group);
		wait_message (buf);
	}
}


/* ARGSUSED0 */
void main_suspend (sig)
	int sig;
{
	Raw (FALSE);
	putchar ('\n');

#ifdef POSIX_JOB_CONTROL
	sigsetmask(0);
#else
	signal(SIGTSTP, SIG_DFL);
#ifdef BSD
	sigsetmask (sigblock(0) & ~(1 << (SIGTSTP -1)));
#endif
#endif

	kill (0, SIGTSTP);

#ifdef POSIX_JOB_CONTROL
	sigemptyset (&main_act.sa_mask);
	main_act.sa_flags = SA_RESTART | SA_RESETHAND;
	main_act.sa_handler = main_suspend;
	sigaction (SIGTSTP, &main_act, 0L);
#else
	signal (SIGTSTP, main_suspend);
#endif

	mail_setup ();
	if (! update) {
		Raw (TRUE);
	}
}


/* ARGSUSED0 */
void select_suspend (sig)
	int sig;
{

	Raw (FALSE);
	putchar ('\n');

#ifdef POSIX_JOB_CONTROL
	sigsetmask(0);
#else
	signal(SIGTSTP, SIG_DFL);
#ifdef BSD
	sigsetmask (sigblock(0) & ~(1 << (SIGTSTP -1)));
#endif
#endif

	kill (0, SIGTSTP);

#ifdef POSIX_JOB_CONTROL
	sigemptyset (&select_act.sa_mask);
	select_act.sa_flags = SA_RESTART | SA_RESETHAND;
	select_act.sa_handler = select_suspend;
	sigaction (SIGTSTP, &select_act, 0L);
#else
	signal (SIGTSTP, select_suspend);
#endif

	if (! update) {
		Raw (TRUE);
	}

#ifndef USE_CLEARSCREEN
	ClearScreen ();
#endif
	mail_setup ();
	group_selection_page ();
}


/* ARGSUSED0 */
void group_suspend (sig)
	int sig;
{
	Raw (FALSE);
	putchar ('\n');

#ifdef POSIX_JOB_CONTROL
	sigsetmask(0);
#else
	signal(SIGTSTP, SIG_DFL);
#ifdef BSD
	sigsetmask (sigblock(0) & ~(1 << (SIGTSTP -1)));
#endif
#endif

	kill (0, SIGTSTP);

#ifdef POSIX_JOB_CONTROL
	sigemptyset (&group_act.sa_mask);
	group_act.sa_flags = SA_RESTART | SA_RESETHAND;
	group_act.sa_handler = group_suspend;
	sigaction (SIGTSTP, &group_act, 0L);
#else
	signal (SIGTSTP, group_suspend);
#endif

	if (! update) {
		Raw (TRUE);
	}
	
#ifndef USE_CLEARSCREEN
	ClearScreen ();
#endif
	mail_setup ();
	show_group_page (glob_group);
}


/* ARGSUSED0 */
void page_suspend (sig)
	int sig;
{

	Raw (FALSE);
	putchar ('\n');

#ifdef POSIX_JOB_CONTROL
	sigsetmask(0);
#else
	signal(SIGTSTP, SIG_DFL);
#ifdef BSD
	sigsetmask (sigblock(0) & ~(1 << (SIGTSTP -1)));
#endif
#endif

	kill (0, SIGTSTP);

#ifdef POSIX_JOB_CONTROL
	sigemptyset (&page_act.sa_mask);
	page_act.sa_flags = SA_RESTART | SA_RESETHAND;
	page_act.sa_handler = page_suspend;
	sigaction (SIGTSTP, &page_act, 0L);
#else
	signal (SIGTSTP, page_suspend);
#endif

	mail_setup ();

	if (! update) {
		Raw (TRUE);
	}
	
#ifndef USE_CLEARSCREEN
	ClearScreen ();
#endif
	redraw_page (glob_respnum, glob_page_group);
}


/* ARGSUSED0 */
void rcfile_suspend (sig)
	int sig;
{
	Raw (FALSE);
	putchar ('\n');

#ifdef POSIX_JOB_CONTROL
	sigsetmask(0);
#else
	signal(SIGTSTP, SIG_DFL);
#ifdef BSD
	sigsetmask (sigblock(0) & ~(1 << (SIGTSTP -1)));
#endif
#endif

	kill (0, SIGTSTP);

#ifdef POSIX_JOB_CONTROL
	sigemptyset (&rcfile_act.sa_mask);
	rcfile_act.sa_flags = SA_RESTART | SA_RESETHAND;
	rcfile_act.sa_handler = rcfile_suspend;
	sigaction (SIGTSTP, &rcfile_act, 0L);
#else
	signal (SIGTSTP, rcfile_suspend);
#endif

	Raw (TRUE);
	show_rcfile_menu ();	
}

#endif /* SIGTSTP */	


#ifdef SIGWINCH

/* ARGSUSED0 */
void art_resize (sig)
	int sig;
{
	char buf[LEN];

	info_message (txt_resizing_window);
	set_win_size (&LINES, &COLS);
	signal (SIGWINCH, art_resize);

	ClearScreen ();
	sprintf (buf, txt_group, glob_art_group);
	wait_message (buf);
}


/* ARGSUSED0 */
void main_resize (sig)
	int sig;
{
	info_message (txt_resizing_window);
	set_win_size (&LINES, &COLS);
	signal (SIGWINCH, main_resize);
}


/* ARGSUSED0 */
void select_resize (sig)
	int sig;
{
	info_message (txt_resizing_window);
	set_win_size (&LINES, &COLS);
	signal (SIGWINCH, select_resize);
	
#ifndef USE_CLEARSCREEN
	ClearScreen ();
#endif
	group_selection_page ();
}


/* ARGSUSED0 */
void group_resize (sig)
	int sig;
{
	info_message (txt_resizing_window);
	set_win_size (&LINES, &COLS);
	signal (SIGWINCH, group_resize);
	
#ifndef USE_CLEARSCREEN
	ClearScreen ();
#endif
	show_group_page (glob_group);
}


/* ARGSUSED0 */
void page_resize (sig)
	int sig;
{
	info_message (txt_resizing_window);
	set_win_size (&LINES, &COLS);
	signal (SIGWINCH, page_resize);
	
#ifndef USE_CLEARSCREEN
	ClearScreen ();
#endif
	redraw_page (glob_respnum, glob_page_group);
}

#endif /* SIGWINCH */

==> tin-1.05.tar.gz: tin-1.05/signal.c <==
/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : signal.c
 *  Author    : R.Skrenta / I.Lea
 *  Created   : 01-04-91
 *  Updated   : 15-10-91
 *  Release   : 1.0
 *  Notes     : signal handlers for different modes and window resizing
 *  Copyright : (c) Copyright 1991 by Rich Skrenta & Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

extern char *glob_art_group;
extern char *glob_group;
extern int glob_respnum;
extern char *glob_page_group;

#ifdef SIGTSTP
int do_sigtstp = 0;
#endif

#ifdef POSIX_JOB_CONTROL
struct sigaction art_act;
struct sigaction group_act;
struct sigaction kill_act;
struct sigaction main_act;
struct sigaction page_act;
struct sigaction rcfile_act;
struct sigaction select_act;
struct sigaction old_act;
#endif


void set_signal_handlers ()
{
	signal (SIGINT, signal_handler);	/* ctrl-C */
	signal (SIGQUIT, signal_handler);	/* ctrl-\ */
	signal (SIGILL, signal_handler);
	signal (SIGBUS, signal_handler);
	signal (SIGSEGV, signal_handler);
 	
	signal (SIGPIPE, SIG_IGN);

#ifdef SIGTSTP
	{
		void (*ptr)();
		ptr = signal (SIGTSTP, SIG_DFL);
		signal (SIGTSTP, ptr);
		if (ptr != SIG_IGN) {
			/*
			 *  SIGTSTP is ignored when starting from shells
			 *  without job-control
			 */
			do_sigtstp = 1; 
			signal (SIGTSTP, main_suspend);
		}
	}
#endif

#ifdef SIGWINCH
	signal (SIGWINCH, main_resize);
#endif
}


void signal_handler (sig)
	int sig;
{
	if (update) {
		Raw (FALSE);
		EndWin ();
		fprintf (stderr, "\n%s: signal handler caught signal %d\n", progname,sig);
		fflush (stderr);
		exit (1);
	} else {
		switch (sig) {
			case SIGINT:
				signal (SIGINT, signal_handler);
				break;
			default:
				Raw (FALSE);
				EndWin ();
				fprintf (stderr, "\n%s: signal handler caught signal %d\n", progname,sig);
				fflush (stderr);
				exit (1);
		}
	}
}


void set_win_size (num_lines, num_cols)
	int *num_lines;
	int *num_cols;
{
#ifdef TIOCGWINSZ
	char buf[64];
	int old_lines, old_cols;
	struct winsize win;

	if (debug) {
		old_lines = *num_lines;
		old_cols = *num_cols;
	}
#endif

	init_screen_array (FALSE);		/* deallocate screen array */

#ifdef TIOCGWINSZ
	if (ioctl (0, TIOCGWINSZ, &win) == 0) {
		if (win.ws_row != 0) {
			*num_lines = win.ws_row - 1;
		}
		if (win.ws_col != 0) {
			*num_cols = win.ws_col;
		}
	}

	if (debug) {
		sprintf (buf, "RESIZED lines %d to %d  cols %d to %d",
			old_lines, *num_lines, old_cols, *num_cols);
		info_message (buf);
	}
#endif

	init_screen_array (TRUE);		/* allocate screen array for resize */

	max_subj = (*num_cols / 2) - 2;
	max_from = (*num_cols - max_subj) - 17;
	RIGHT_POS = *num_cols - 18;
	MORE_POS  = *num_cols - 15;
	NOTESLINES = *num_lines - INDEX_TOP - 1;
	if (NOTESLINES <= 0) {
		NOTESLINES = 1;
	}
}


#ifdef SIGTSTP

/* ARGSUSED0 */
void art_suspend (sig)
	int sig;
{
	char buf[LEN];
	
	Raw (FALSE);
	putchar ('\n');

#ifdef POSIX_JOB_CONTROL
	sigsetmask(0);
#else
	signal(SIGTSTP, SIG_DFL);
#ifdef BSD
	sigsetmask (sigblock(0) & ~(1 << (SIGTSTP -1)));
#endif
#endif

	kill (0, SIGTSTP);

#ifdef POSIX_JOB_CONTROL
	sigemptyset (&art_act.sa_mask);
	art_act.sa_flags = SA_RESTART | SA_RESETHAND;
	art_act.sa_handler = art_suspend;
	sigaction (SIGTSTP, &art_act, 0L);
#else
	signal (SIGTSTP, art_suspend);
#endif

	if (! update) {
		mail_setup ();
	
		Raw (TRUE);

		ClearScreen ();
		sprintf (buf, txt_group, glob_art_group);
		wait_message (buf);
	}
}


/* ARGSUSED0 */
void main_suspend (sig)
	int sig;
{
	Raw (FALSE);
	putchar ('\n');

#ifdef POSIX_JOB_CONTROL
	sigsetmask(0);
#else
	signal(SIGTSTP, SIG_DFL);
#ifdef BSD
	sigsetmask (sigblock(0) & ~(1 << (SIGTSTP -1)));
#endif
#endif

	kill (0, SIGTSTP);

#ifdef POSIX_JOB_CONTROL
	sigemptyset (&main_act.sa_mask);
	main_act.sa_flags = SA_RESTART | SA_RESETHAND;
	main_act.sa_handler = main_suspend;
	sigaction (SIGTSTP, &main_act, 0L);
#else
	signal (SIGTSTP, main_suspend);
#endif

	mail_setup ();
	if (! update) {
		Raw (TRUE);
	}
}


/* ARGSUSED0 */
void select_suspend (sig)
	int sig;
{

	Raw (FALSE);

#ifdef POSIX_JOB_CONTROL
	sigsetmask(0);
#else
	signal(SIGTSTP, SIG_DFL);
#ifdef BSD
	sigsetmask (sigblock(0) & ~(1 << (SIGTSTP -1)));
#endif
#endif

	kill (0, SIGTSTP);

#ifdef POSIX_JOB_CONTROL
	sigemptyset (&select_act.sa_mask);
	select_act.sa_flags = SA_RESTART | SA_RESETHAND;
	select_act.sa_handler = select_suspend;
	sigaction (SIGTSTP, &select_act, 0L);
#else
	signal (SIGTSTP, select_suspend);
#endif

	if (! update) {
		Raw (TRUE);
	}

#ifndef USE_CLEARSCREEN
	ClearScreen ();
#endif
	mail_setup ();
	group_selection_page ();
}


/* ARGSUSED0 */
void group_suspend (sig)
	int sig;
{
	Raw (FALSE);
	putchar ('\n');

#ifdef POSIX_JOB_CONTROL
	sigsetmask(0);
#else
	signal(SIGTSTP, SIG_DFL);
#ifdef BSD
	sigsetmask (sigblock(0) & ~(1 << (SIGTSTP -1)));
#endif
#endif

	kill (0, SIGTSTP);

#ifdef POSIX_JOB_CONTROL
	sigemptyset (&group_act.sa_mask);
	group_act.sa_flags = SA_RESTART | SA_RESETHAND;
	group_act.sa_handler = group_suspend;
	sigaction (SIGTSTP, &group_act, 0L);
#else
	signal (SIGTSTP, group_suspend);
#endif

	if (! update) {
		Raw (TRUE);
	}
	
#ifndef USE_CLEARSCREEN
	ClearScreen ();
#endif
	mail_setup ();
	show_group_page (glob_group);
}


/* ARGSUSED0 */
void page_suspend (sig)
	int sig;
{

	Raw (FALSE);
	putchar ('\n');

#ifdef POSIX_JOB_CONTROL
	sigsetmask(0);
#else
	signal(SIGTSTP, SIG_DFL);
#ifdef BSD
	sigsetmask (sigblock(0) & ~(1 << (SIGTSTP -1)));
#endif
#endif

	kill (0, SIGTSTP);

#ifdef POSIX_JOB_CONTROL
	sigemptyset (&page_act.sa_mask);
	page_act.sa_flags = SA_RESTART | SA_RESETHAND;
	page_act.sa_handler = page_suspend;
	sigaction (SIGTSTP, &page_act, 0L);
#else
	signal (SIGTSTP, page_suspend);
#endif

	mail_setup ();

	if (! update) {
		Raw (TRUE);
	}
	
#ifndef USE_CLEARSCREEN
	ClearScreen ();
#endif
	redraw_page (glob_respnum, glob_page_group);
}


/* ARGSUSED0 */
void rcfile_suspend (sig)
	int sig;
{
	Raw (FALSE);
	putchar ('\n');

#ifdef POSIX_JOB_CONTROL
	sigsetmask(0);
#else
	signal(SIGTSTP, SIG_DFL);
#ifdef BSD
	sigsetmask (sigblock(0) & ~(1 << (SIGTSTP -1)));
#endif
#endif

	kill (0, SIGTSTP);

#ifdef POSIX_JOB_CONTROL
	sigemptyset (&rcfile_act.sa_mask);
	rcfile_act.sa_flags = SA_RESTART | SA_RESETHAND;
	rcfile_act.sa_handler = rcfile_suspend;
	sigaction (SIGTSTP, &rcfile_act, 0L);
#else
	signal (SIGTSTP, rcfile_suspend);
#endif

	Raw (TRUE);
	show_rcfile_menu ();	
}

#endif /* SIGTSTP */	


#ifdef SIGWINCH

/* ARGSUSED0 */
void art_resize (sig)
	int sig;
{
	char buf[LEN];

	info_message (txt_resizing_window);
	set_win_size (&LINES, &COLS);
	signal (SIGWINCH, art_resize);

	ClearScreen ();
	sprintf (buf, txt_group, glob_art_group);
	wait_message (buf);
}


/* ARGSUSED0 */
void main_resize (sig)
	int sig;
{
	info_message (txt_resizing_window);
	set_win_size (&LINES, &COLS);
	signal (SIGWINCH, main_resize);
}


/* ARGSUSED0 */
void select_resize (sig)
	int sig;
{
	info_message (txt_resizing_window);
	set_win_size (&LINES, &COLS);
	signal (SIGWINCH, select_resize);
	
#ifndef USE_CLEARSCREEN
	ClearScreen ();
#endif
	group_selection_page ();
}


/* ARGSUSED0 */
void group_resize (sig)
	int sig;
{
	info_message (txt_resizing_window);
	set_win_size (&LINES, &COLS);
	signal (SIGWINCH, group_resize);
	
#ifndef USE_CLEARSCREEN
	ClearScreen ();
#endif
	show_group_page (glob_group);
}


/* ARGSUSED0 */
void page_resize (sig)
	int sig;
{
	info_message (txt_resizing_window);
	set_win_size (&LINES, &COLS);
	signal (SIGWINCH, page_resize);
	
#ifndef USE_CLEARSCREEN
	ClearScreen ();
#endif
	redraw_page (glob_respnum, glob_page_group);
}

#endif /* SIGWINCH */
